/*--------------------------------------------------------------------
 * $Id: uvspec.h,v 1.133 2006/12/17 23:45:52 bernhard Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

#ifndef __uvspec_h
#define __uvspec_h

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "mystic.h"



#define VERSION "1.0"

/* Indices for the reading of the atmosphere files */
#define ATMFILE_ZD       0
#define ATMFILE_PRESS    1
#define ATMFILE_TEMPER   2
#define ATMFILE_AIR      3
#define ATMFILE_O3       4
#define ATMFILE_O2       5
#define ATMFILE_H2O      6
#define ATMFILE_CO2      7
#define ATMFILE_NO2      8

/* Atmospheric molecules that may be included in spectral calculations */
#define MOL_NN        9
#define MOL_AIR       0
#define MOL_O3        1
#define MOL_O2        2
#define MOL_H2O       3
#define MOL_CO2       4
#define MOL_NO2       5
#define MOL_BRO       6
#define MOL_OCLO      7
#define MOL_HCHO      8

/* Possible units of total molecule column */
#define CM_2          1
#define DU            2

/* Possible units of density profiles */
#define CM_3          1    /* per cubic centimeter */
#define M_3           2    /* per cubic meter      */
#define MMR           3    /* mass mixing ratio    */
#define VMR           4    /* volumn mixing ratio  */
#define RH            5    /* relative humidity    */


#define FN_NN               30  /* Number of general files, increase 
				   when adding new general files */

#define FN_PATH               0
#define FN_ALBEDO             1
#define FN_ALBEDO_LIB_PATH    2
#define FN_ALBEDO_LIB_WVL     3
#define FN_ALBEDO_MAP         4
#define FN_ALTITUDE_MAP       5
#define FN_ATMOSPHERE         6 
#define FN_EXTRATERRESTRIAL   7 
#define FN_SZA                8
#define FN_WLTRANS            9
#define FN_WLBANDS           10 
#define FN_WCLOUD            11
#define FN_WCLOUD_FILES      12 
#define FN_WCLOUD_PROP       13
#define FN_WCLOUD_IPA        14
#define FN_WCLOUD_IPA_3D     15
#define FN_ICLOUD            16
#define FN_ICLOUD_FILES      17 
#define FN_ICLOUD_PROP       18 
#define FN_ICLOUD_IPA        19 
#define FN_CLOUD_FRACTION    20
#define FN_SLITFUNCTION      21
#define FN_FILTERFUNCTION    22
#define FN_SPLINE            23
#define FN_RAYLEIGH          24
#define FN_MOLABS            25
#define FN_RPV               26
#define FN_REFIND            27
#define FN_CK_GENERIC        28
#define FN_RH                29


#define FN_AER_NN            7  /* Number of aerosol related files, increase 
				   when adding new aerosol files */
#define FN_AER_TAU           0
#define FN_AER_GG            1
#define FN_AER_SSA           2
#define FN_AER_MOM           3
#define FN_AER_REF           4
#define FN_AER_SIZ           5
#define FN_AER_FILES         6

#define FN_MC_NN            12  /* Number of mystic related files, increase 
				   when adding new mystic files */
#define FN_MC_PHOTONS        0
#define FN_MC_MOMCLD         1
#define FN_MC_MOMTRNC        2
#define FN_MC_PCLD           3
#define FN_MC_BASENAME       4
#define FN_MC_ALBEDO         5
#define FN_MC_ELEVATION      6
#define FN_MC_RPV            7
#define FN_MC_RPVTYPE        8
#define FN_MC_AMBRALS        9
#define FN_MC_WCLOUD        10
#define FN_MC_ICLOUD        11


#define CK_NONE         0
#define CK_KATO         1
#define CK_KATO2        2
#define CK_KATO2_96     3
#define CK_FU           4
#define CK_AVHRR_KRATZ  5
#define CK_GENERIC      6
#define CK_LOWTRAN      7

#define WLGRID_NONE    0
#define WLGRID_CK      1
#define WLGRID_USER    2
#define WLGRID_BANDS   3
#define WLGRID_MOLABS  4
#define WLGRID_UVSPEC  5

#define SRC_NONE    0
#define SRC_SOLAR   1
#define SRC_THERMAL 2

#define RAYLEIGH_BODHAINE  1
#define RAYLEIGH_NICOLET   2

#define O3_MOLINA    1
#define O3_DAUMONT   2
#define O3_BASSPAUR  3    

#define CO2_LEWIS    1

#define O2_DEFAULT   1

#define NO2_GOME     1

#define BRO_IASB     1

#define OCLO_WAHNER  1

#define HCHO_IASB  1

#define RAYLEIGH_NONE      0
#define RAYLEIGH_CALC      1
#define RAYLEIGH_FILE_MONO 2
#define RAYLEIGH_FILE_SPEC 3

#define MOLABS_NONE      0
#define MOLABS_CALC      1
#define MOLABS_FILE_MONO 2
#define MOLABS_FILE_SPEC 3

#define MX_NN  9  
#define MX_O2  0
#define MX_CO2 1
#define MX_H2O 2
#define MX_CH4 3
#define MX_N2O 4
#define MX_NO2 5
#define MX_F11 6
#define MX_F12 7
#define MX_F22 8

#define CLD_ZD     0    
#define CLD_LWC    1
#define CLD_EFFR   2

#define BRDF_NONE   0
#define BRDF_RPV    1  /* don't change these numbers as they are used by Fortran code */
#define BRDF_CM     2  /* which of course has no access to this header file           */
#define BRDF_RPV2D  3
#define BRDF_CM2D   4

#define DISORT      1
#define SDISORT     2
#define TWOSTR      3
#define SUCORD      4
#define MONTECARLO  5
#define POLRADTRAN  6
#define DISORT2     7
#define SPSDISORT   8
#define QDISORT     9
#define TZS        10
#define SSS        11
#define SSSI       12
#define SSS2       13
#define NULLSOLVER 14

#define MOMCLD_NONE       0
#define MOMCLD_WATER      1
#define MOMCLD_ICE        2
#define MOMCLD_FILE       3

#define WCPROP_NONE         0   /* no water cloud                            */
#define WCPROP_HU           1   /* "wc_file" and Hu and Stamnes [1993]       */
#define WCPROP_ECHAM4       2   /* "wc_file" and ECHAM4                      */
#define WCPROP_MIE          3   /* "wc_file" and "wc_properties mie"         */
#define WCPROP_FILE         4   /* "wc_file" and "wc_properties <filename>"  */
#define WCPROP_EXPLICIT     5   /* "wc_files"                                */

#define ICPROP_NONE         0   /* no ice cloud                              */
#define ICPROP_FU           1   /* "ic_file" and Fu et al. [1996/98]         */
#define ICPROP_ECHAM4       2   /* "ic_file" and ECHAM4                      */
#define ICPROP_KEY          3   /* "ic_file" and Key et al. [2002]           */
#define ICPROP_YANG         4   /* "ic_file" and Yang/Key/Mayer              */
#define ICPROP_MIE          5   /* "ic_file" and "ic_properties mie"         */
#define ICPROP_FILE         6   /* "ic_file" and "ic_properties <filename>"  */
#define ICPROP_EXPLICIT     7   /* "ic_files"                                */
#define ICPROP_BAUM         8   /* "ic_properties baum"                      */
#define ICPROP_BAUMDETAILED 9   /* "ic_properties baum_detailed"             */

#define SATURATION_NONE     0
#define SATURATION_CLOUD    1
#define SATURATION_IPA      2

#define PROCESS_NONE        0
#define PROCESS_SUM         1
#define PROCESS_RGB         2
#define PROCESS_RGBNORM     3
#define PROCESS_INT         4

#define OUTPUT_PER_NM      0
#define OUTPUT_PER_CMm1    1
#define OUTPUT_PER_CK_BAND 2

#define NO_HEAT       0   /* no heating rate calculation                                               */
#define HEAT_LOCAL    1   /* heating rates from (k_abs * actinic fluxes)                               */
#define HEAT_LAYER_FD 2   /* heating rates calculated with Forward  Difference of the flux over layers */
#define HEAT_LAYER_CD 3   /* heating rates calculated with Centered Difference of the flux over layers */

/* main switch for interpolation onto zout and altitude levels */
#define Z_INTERPOLATE_LOGLIN  0      /* no interpolation                   */
#define Z_INTERPOLATE_LINMIX  1      /* linear mixing ratio interpolation  */
#define Z_INTERPOLATE_SPLINE  2      /* spline interpolation               */

/* assumptions for interpolation of profiles,   */
/* calculation of effective densities for       */
/* optical properties and columns               */
#define INTERP_METHOD_SPLINE     0  /* spline interpolation (columns not consistent)               */
#define INTERP_METHOD_LINEAR     1  /* linear profile                                              */
#define INTERP_METHOD_LOG        2  /* logarithmic profile                                         */
#define INTERP_METHOD_LINMIX     3  /* linear mixing ratio (only for gas densities other than air) */
#define INTERP_METHOD_LOG_SPLINE 4  /* first take logarithm, than spline (columns not consistent)  */

#define NO_ZOUT_INTERPOLATE    0  /* used for zout_interpolate */ 
#define ZOUT_INTERPOLATE       1  /* used for zout_interpolate */

#define ZOUT_MYSTIC  -999
#define ZOUT_SURFACE -666
#define ZOUT_TOA      666

#define OUTPUT_USER_NONE         0
#define OUTPUT_USER_WAVE         1
#define OUTPUT_USER_WAVENUMBER   2
#define OUTPUT_USER_ZOUT_SUR     3
#define OUTPUT_USER_ZOUT_SEA     4
#define OUTPUT_USER_P            5
#define OUTPUT_USER_T            6
#define OUTPUT_USER_T_D          7
#define OUTPUT_USER_THETA        8
#define OUTPUT_USER_THETA_E      9
#define OUTPUT_USER_EDIR        10
#define OUTPUT_USER_EGLO        11
#define OUTPUT_USER_EDN         12
#define OUTPUT_USER_EUP         13
#define OUTPUT_USER_ENET        14
#define OUTPUT_USER_ESUM        15
#define OUTPUT_USER_FDIR        16
#define OUTPUT_USER_FGLO        17
#define OUTPUT_USER_FDN         18
#define OUTPUT_USER_FUP         19
#define OUTPUT_USER_F           20
#define OUTPUT_USER_UDIR        21
#define OUTPUT_USER_UGLO        22
#define OUTPUT_USER_UDN         23
#define OUTPUT_USER_UUP         24
#define OUTPUT_USER_U           25
#define OUTPUT_USER_ALB         26
#define OUTPUT_USER_HEAT        27
#define OUTPUT_USER_W_RAD       28
#define OUTPUT_USER_N           29
#define OUTPUT_USER_RHO         30
#define OUTPUT_USER_VMR         31
#define OUTPUT_USER_MMR         32

#define ZOUT_ABOVE_SUR         0
#define ZOUT_ABOVE_SEA         1

#define OUTCAL_ABSOLUTE        0
#define OUTCAL_THERMAL         1
#define OUTCAL_TRANSMITTANCE   2
#define OUTCAL_REFLECTIVITY    3
#define OUTCAL_BRIGHTNESS      4

#define GSPEC_NONE             0
#define GSPEC_EXT_GG           1
#define GSPEC_EXT_REFF         2
#define GSPEC_LWC_REFF         3

#define WSPEC_NONE             0
#define WSPEC_SSA              1

#define ISCCP_WATER     0
#define ISCCP_ICE       1

#define MAXAWVN        47

#define FALSE           0
#define NO              0
#define TRUE            1
#define YES             1

#define ALBEDO_CONSTANT     0 
#define ALBEDO_FROM_FILE    1
#define ALBEDO_IGBP_LIBRARY 2
#define ALBEDO_USER_LIBRARY 3

#define ALT_NOT_DEFINED  0
#define ALT_DIRECT_INPUT 1
#define ALT_FROM_MAP     2
#define ALT_FROM_ATM     3

/* minimum number of photons for each band*/
#define MIN_MCPHOTONS 100

/* physical constants */
#define c_p      1004.0   /* specific heat of dry air at constant pressure (J kg-1 K-1) */
#define c_v       717.0   /* specific heat of dry air at constant volumn   (J kg-1 K-1) */


/**********************************************/
/*            structure definitions           */
/**********************************************/

/**************************/
/* Monte Carlo STRUCTURES */
/**************************/

/* Monte Carlo backward structure */

typedef struct {
  int yes;  /* flag if backward is desired */

  int islower;   /* x-index of lower left  sample pixel */
  int jslower;   /* y-index of lower left  sample pixel */
  int isupper;   /* x-index of upper right sample pixel */
  int jsupper;   /* y-index of upper right sample pixel */

  int writeallpixels;
} mc_backward_struct;

/* Monte Carlo input structure */

typedef struct {
  int     photons;

  int     Nx_sample;
  int     Ny_sample;

  double  dx_sample;
  double  dy_sample;

  int     Nr;   /* number of radii, for I3RC case 7 */
  int     Nt;   /* number of times, for I3RC case 7 */
  float   dt;   /* time interval,   for I3RC case 7 */

  int     passback3D;
  int     ipa;            /* MYSTIC in IPA mode                         */
  int     escape;         /* escape probabilities for radiances         */
  float   truncate;       /* truncate phase function                    */

  int     spherical;      /* spherical flag                             */

  int     bcond;          /* boundary conditions                        */

  int     reflectalways;  /* reflect each photon and weight with albedo */

  int     absorption;     /* calculate absorption for each box          */ 

  int     momcld;
  char    **filename;

  int     azimuth;        /* new or old azimuth convention              */

  int     std;            /* calculate standard deviation               */
  int     randomseed;     /* seed for the random number generator       */

  mc_backward_struct backward;  /* information for backward MYSTIC      */
} mc_inp_struct;

/* Monte Carlo output structure */
/* (MYSTIC uses profiles in reverted order (the lowest level is 0)) */

typedef struct {
  float       *dtair;           /* free'ed */
  float       *dtmol;           /* free'ed */
  float       *dtaer;           /* free'ed */
  float       *dtcld;           /* free'ed */
  float       *dtice;           /* free'ed */
  float       *omaer;           /* free'ed */
  float       *omcld;           /* free'ed */
  float       *omice;           /* free'ed */

  float       *g1aer;
  float       *g2aer;
  float       *ffaer;

  float       *g1cld;
  float       *g2cld;
  float       *ffcld;

  float       *g1ice;
  float       *g2ice;
  float       *ffice;

  float       *refind;

  float       **momaer;
  int         *nmomaer;

  float       *z;
  float       *temper;

  float       *recld;
  float       *lwccld;

  sample_struct sample;   /* 3d sampling information  */
} mc_out_struct;


/***********************************************/
/* Raditive Transfer Equation STRUCTURE        */
/* See disort.doc for description of variables */
/***********************************************/

typedef struct {
  int     deltam;
  int     fisot;
  int     ichap;  
  int     nrefrac;  
  int     nstr;  
  int     nscat;
  int     maxphi;
  int     maxumu;
  int     nphi;
  int     numu;
  int     solver;
  int     nprndis;
  int     reverse;
  int*    prndis;
  int     pol_aziorder;
  int     pol_nstokes;
  char*   pol_quad_type;
  char    pol_stokes[4];
  int     pol_src_code;
  double  pol_max_delta_tau;
  float*  phi;
  float*  umu;
  mc_inp_struct mc;
} rte_struct;


/* Cloud output STRUCTURE */

typedef struct {
  /* quantities as defined by the user */
  float *lwc;        /* z-profile of the liquid water content */
  float *effr;       /* z-profile of the effective radius     */

  /* layer quantities */
  float *lwc_layer;  /* z-profile of the liquid water content */
  float *effr_layer; /* z-profile of the effective radius     */

  int   alloc;       /* flag, if memory is allocated                 */
} cld_microphys_struct;


/********************************************************/
/* optical properties of aerosol, water, and ice clouds */
/********************************************************/

typedef struct {
  float **dtau;     /* layer optical thickness [wavelength][layer]         */

  float **ff;       /* f  for double HG [wavelength][layer]                */
  float **g1;       /* g1 for double HG [wavelength][layer]                */
  float **g2;       /* g2 for double HG [wavelength][layer]                */

  float **ssa;      /* layer single scattering albedo [wavelength][layer]  */
  float **fdel;     /* delta-m scaling factor                              */

  int   nlam;       /* number of wavelengths                               */
	
  float ***moment;  /* moments of the phase function for each              */
                    /* wavelength and layer                                */

  int   **nmom;     /* number of moments for each wavelength and layer     */

  int alloc;        /* flag, if memory is allocated;                       */
} optprop_struct;


typedef struct {
  /* the following quantities are defined for each wavelength, */
  /* on an equidistant grid of effective radii,                */
  /* r0, r0+dr, r0+2*dr, ..., r0 + (nreff-1)*dr                */

  double **extinc;    /* extinction coefficient             */
  double **albedo;    /* single scattering albedo           */
  float  ***legen;    /* Legendre moments                   */
  int    **nleg;      /* number of Legendre moments         */
  double  *r0;        /* start radius for each wavelength   */
  double  *dr;        /* wavelength step                    */
  int     *nreff;     /* number of effective radii          */ 
   
  int   alloc;        /* flag, if basic memory is allocated */
  int   allocdetail;  /* flag, if full memory is allocated  */
} cld_ssprop_struct;

/* Cloud output STRUCTURE */

typedef struct {

  int   occurrence; /* are there any clouds at all? */
  float *zd;        /* level altitudes                              */
  int   nlev;       /* number of levels                             */

  cld_microphys_struct microphys;  /* microphysical properties     */
  optprop_struct       optprop;    /* optical properties           */
  cld_ssprop_struct    ssprop;     /* single scattering properties */

  cld3d_out_struct     threed;     /* 3d water cloud structure     */

  int newsiz;      /* flag for the Fortran function wcloud() */

  double *z3D;     /* checking of the 3D grid */ 
  int    Nz3D;
  
} cld_out_struct;

/* Cloud input STRUCTURE */

typedef struct {
  int     layer;

  float   tau;  
  float   gg;  
  float   ssa;  

  float   tau550;  

  float scale_gg;
  float scale_ssa;

  int properties;
  int unscaled;
  int fu2yang;
  int interpolate;
  int habit;
  int wv_saturation;
  float wv_sat_ratio;
} cld_inp_struct;

/**********************/
/* Aerosol structures */
/**********************/

/* Aerosol input STRUCTURE */

typedef struct {
  int     standard;  /* Shettle standard aerosol defined    */
  int     spec;      /* specific aerosol properties defined */

  char **filename;   /* array of filenames */
  int haze;
  int vulcan;
  int seasn;
  float alpha;
  float beta;
  float visibility;
  
  float re;          /* refractive index            */
  float im;          /* (independent of wavelength) */

  float tau550;

  float scale_ssa;
  float scale_tau;
  float set_gg;
  float set_ssa;
  float set_tau;
} aer_inp_struct;

/* Aerosol output STRUCTURE */

typedef struct {
  float   visibility;

  float *zd;       /* level altitudes                  */
  int   nlev;      /* number of levels                 */

  float *zd_alt;       /* level altitudes, starting at 'altitude'   */
  int   nlev_alt;      /* number of levels, starting at 'altitude'  */

  /* optical properties profile */
  optprop_struct optprop;
  optprop_struct *optprop_ipa;

  float   *aer_pro;

  double   *xsiz;    /* size distribution              */
  double   *ysiz;
  int      nsiz;

  float   *re_t;     /* real and imaginary part of the */
  float   *im_t;     /* refractive index               */
} aer_out_struct;

/*********************/
/* Albedo structures */
/*********************/

/* albedo input structure */

typedef struct {
  int    source;    /* where to get the albedo from (constant, file, library) */
  int    surface;   /* reference number for spectral albedo type inside a library */
  float  albedo;    /* constant input albedo value */
  int    map;       /* specifies, if the surface type is picked up in a map */
  float  map_lat_min; /* specifies the size of the map */
  float  map_lat_max; /*            "                  */
  float  map_lon_min; /*            "                  */
  float  map_lon_max; /*            "                  */ 
} alb_inp_struct;

/* albedo output structure */

typedef struct {

  int    source;    /* where to get the albedo from (constant, file, library) */
  int    surface;   /* reference number for spectral albedo type inside a library */
  float  albedo;    /* constant input albedo value */
  int    map;       /* specifies, if the surface type is picked up in a map */
  float  map_lat_min; /* specifies the size of the map */
  float  map_lat_max; /*            "                  */
  float  map_lon_min; /*            "                  */
  float  map_lon_max; /*            "                  */ 

  int    file_nlambda_albedo;  /* number of elements in the albedo file   */
  float *file_albedo;          /* albedo on the spectral grid of the file */
  float *file_lambda_albedo;   /* wavelength grid of the albedo file      */

  float *albedo_t;             /* albedo on the actual transmission grid  */
  int    nlambda_albedo;

} alb_out_struct;

/* RPV (Rahman, Pinty, Verstraete) structure */

typedef struct {
  float rho0;
  float k;
  float theta;
  float sigma; /* sigma, t1, t2: Deguenther and Meerkoetter [2000]; */
  float t1;    /* snow; includes forward peak.                      */
  float t2;
  float scale; /* optional scaling factor for the BRDF, to adjust albedo */
} rpv_inp_struct;


typedef struct {
  float iso;  /* isotropic BRDF parameter  */
  float vol;  /* volumetric BRDF parameter */
  float geo;  /* geometric BRDF parameter  */
} ambrals_inp_struct;


/* Cox and Munk structure */

typedef struct {
  float u10;
  float pcl;
  float sal;
} cm_inp_struct;


/* RPV (Rahman, Pinty, Verstraete) structure */
/* to store wavelength dependent BRDF        */
typedef struct {
  int   n;

  float *lambda;
  float *rho0;     /* input wavelength grid    */
  float *k;
  float *theta;
  float *scale;

  float *rho0_t;   /* internal wavelength grid */
  float *k_t;
  float *theta_t;
  float *scale_t;

} rpv_out_struct;

/* Array of RPV (Rahman, Pinty, Verstraete) structures */
/* for different surfaces, addressed by a label        */
typedef struct {
  int   n;

  char  **label;
  char  **filename;

  rpv_out_struct *rpv;
} surfaces_out_struct;


typedef struct {
  float tautot;
  int lctop;
  int type;
  float ref;
} sssi_out_struct;


/*************************/
/* Wavelength structures */
/*************************/

/* Wavelength input grid STRUCTURE */

typedef struct {
  float start;
  float end;

  int start_index;
  int end_index;
} wl_inp_struct;

/* Wavelength output grid STRUCTURE */

typedef struct {
  float start;
  float end;

  int   type;

  int   nlambda_h;
  int   nlambda_s;
  int   nlambda_t;

  int   nlambda_h_print3D;

  float *lambda_h;
  float *lambda_s;
  float *lambda_t;

  int   *map_e2h;

  int   ignore_solar_file;

  float *fbeam;
  float *filter;

  float *wvnmlo_t;
  float *wvnmhi_t;

  float *wvnmlo_h;
  float *wvnmhi_h;

  int  nlambda_rte_lower;
  int  nlambda_rte_upper;

  float *pfraction;
} wl_out_struct;


/**********************/
/* Altitude structure */
/**********************/

typedef struct {
  int    source;        /* where to get the altitude from (direct input or map) */
  float  altitude;      /* surface altitude over sea level */
  float  altitude_dz;   /* fakes an increased vertical resolution, profiles are interpolated */
  float  map_lat_min;
  float  map_lat_max;
  float  map_lon_min;
  float  map_lon_max;
} alt_inp_struct;

typedef struct {
  int    source;        /* where to get the altitude from (direct input, map, or atmosphere) */  
  float  altitude;      /* Same as altitude from input structure is set.
			   Otherwise obtained from atmosphere input file */
  float  map_lat_min;
  float  map_lat_max;
  float  map_lon_min;
  float  map_lon_max;
} alt_out_struct;

/*************************/
/* Atmosphere structures */
/*************************/

/* Atmosphere input data STRUCTURE */

typedef struct {
  char **filename;      /* Filename of density profile, input using dens_file  */
  char  *rs_filename;   /* Filename of radiosonde data, input using radiosonde */
  int    n_rs_gas;      /* number of gases, which are given by radiosonde file */
  int   *rs_gas;        /* MOL-number for gases given by radiosonde data */
  int   *rs_unit;       /* specifies unit, in which radiosonde data is given in */  
  float *column;        /* Total column of a profile, input using dens_column  */
/*   float *scale_factor; /\* Factor to scale profile with, calculated           *\/ */
  int   *unit_column;   /* Unit of the column value, input using dens_column   */
  int   *unit_profile;  /* Unit of the profile, input using dens_file          */
  float *mol_mass;      /* molecular mass of one molucule in u                 */

  float *zout_sur;      /* output altitudes, relative to the surface           */
  float *zout_sea;      /* output altitudes, relative to sea level             */
  int    nzout;         /* number of output altitudes                          */
  int    zout_source;   /* input switch */

  float  sza;
  float  phi0;
  int    zout_interpolate; /* switch for zout-level interpolation */ 
  int    z_interpolate;    /* group specification of the interpolation methods */
                           /* of the atmospheric profiles                      */
  int    interpol_method_press;  /* individual z_interpolation method */
  int    interpol_method_temper; /* individual z_interpolation method */
  int    interpol_method_refind; /* individual z_interpolation method */
  int   *interpol_method_gas;    /* individual z_interpolation methods */ 

} atm_inp_struct;

/* Atmosphere output data STRUCTURE */

typedef struct {

  float *press;
  float *refind;
  float *temper;
  float *temper_avg;     /* average temperature temper_avg[lc] in the layer from z[lc] to z[lc+1]  */ 

  float **dens;          /* density profiles */
  float **dens_avg;      /* layer average density dens_avg[lc] in the layer from z[lc] to z[lc+1]  */
  float ***denstab;      /* density profiles as function of altitude and sza */
  float ***denstab_avg;  /* layer average density profiles as function of altitude and sza */
  float **denstab_amf;   /* density profile as function of altitude and sza, for sdisort */
  float **dens_zout;     /* gas number densities  on the zout - grid [MOL_GAS][lc] */
  // float *dens_air;       /* density of air on the zout - gird */
  // float *dens_h2o_zout;  /* water-vapour density  on the zout - grid */
  float *press_zout;     /* pressure              on the zout - grid */
  float *temper_zout;    /* temperature           on the zout - grid */
  float *temper_d_zout;  /* dewpoint temperature  on the zout - grid */
  float *theta_zout;     /* potential temperature on the zout - grid */
  float *theta_e_zout;   /* equivalent potential temperature on the zout - grid */

  float *sza_denstab;    /* Ary for the sza in denstab */
  
  int    denstab_id;      /* Density id for denstab, MOL_* */
  int    nsza_denstab;    /* Number of sza in denstab */

} atm_microphys_struct;


typedef struct {
  
  double ***tau_rayleigh_t;    /* wavelength-dependent Rayleigh scattering optical depth   */
  double ***tau_molabs_t;      /* wavelength-dependent molecular absorption optical depth  */
  double ***tau_molabs_md_t;   /* wavelength-dependent molecular absorption optical depth  */
			       /* with density-matrix absorption subtracted. For AMF       */
			       /* calculations                                             */
                               /* format: tau_molabs_t[lc][iv][iq]                         */
                               /* ??? this is not consistent with tau_molabs_user_t[][]    */
                               /* ??? and should be changed accordingly                    */
  double ****tau_molabs_t_ipa; /* h2o saturation inside of cloud dependent on IPA-clouds   */
                               /* format: tau_molabs_t_ipa[ic][lc][iv][iq]                 */

  /* The following are explicitely defined by the user and do not depend on subband iq     */
  /* they are copied to the above structures which are used as input to the solvers        */
  /* ATTENTION: These structures are not redistributed and must therefore not be used      */
  /* after the call to setup_redistribute(); optical_properties relies only on the above   */
  /* quantities.                                                                           */

  double  *tau_rayleigh_user;   /* monochromatic Rayleigh scattering optical depth         */
  double **tau_rayleigh_user_t; /* wavelength-dependend Rayleigh scattering optical depth  */
                                /* format: tau_rayleigh_user_t[iv][lc]                     */
  double  *tau_molabs_user;     /* monochromatic molecular absorption optical depth        */
  double **tau_molabs_user_t;   /* wavelength-dependend molecular absorption optical depth */
                                /* format: tau_molabs_user_t[iv][lc]                       */
} atm_optprop_struct;


typedef struct {
  char* pol_scat_files;
  float *cdento;

  float *temper_out;

  atm_microphys_struct microphys;
  atm_optprop_struct   optprop;

  int   molabs;               /* molecular absorption flag         */
  int   rayleigh;             /* Rayleigh scattering flag          */

  float *zd;
  int   nlev;
  int   nlyr;

  float *zout_sur;     /* output altitudes, relative to the surface */
  float *zout_sea;     /* output altitudes, relative to sea level   */
  int    nzout;        /* number of output altitudes                */

  float *zd_common;
  int    nlev_common;
  int    nlyr_common;

  double **wght_t;        /* ck quadrature weight                   */
  double ***ipa_wght_t;   /* independent pixel ck quadrature weight */
  int    *nq_t;           /* number of ck quadrature weights        */

  float *file_phi0;
  float *file_sza;
  float *file_lambda_sza;
  float *sza_t;
  float *phi0_t;

  int    file_nlambda_sza;
  int    nlambda_sza;

  int    nmom;  

  int nthreed;   /* number of 3D layers   */
  int *threed;   /* flag if a layer is 3D */
  
  int Nxcld;     /* 3D horizontal cloud grid */
  int Nycld;     /* 3D horizontal cloud grid */
  int Nzcld;     /* 3D horizontal cloud grid */
} atm_out_struct;

/*******************************************************************/
/* The ck structure holds all the information required for         */
/* a correlated-k approximation.                                   */
/* Components:                                                     */
/*    scheme               CK_KATO, CK_FU                          */
/*    wvlc  [1 ... n_wvl]  central wavelength [nm]                 */                   
/*    wvnlo [1 ... n_wvl]  lower wavenumber [cm-1] for each band   */    
/*    wvnhi [1 ... n_wvl]  upper wavenumber [cm-1] for each band   */    
/*    n_wvl                number of wavelength bands              */
/*    comp  [1 ... n_comp][1 ... n_wvl]                            */
/*                         monochromatic cross sections bands      */
/*                         which don't require c-k but only a      */
/*                         monochromatic calculation               */
/*                         (e.g. ozone in the UV/VIS)              */
/*    wght  [1 ... n_comp][1 ... n_wvl]                            */
/*                         number of quadrature points             */
/*******************************************************************/

typedef struct {   /* correlated-k structure */
  int scheme;

  double *wvlc;
  double *wvnlo;
  double *wvnhi;
  int n_wvl;

  double **comp;
  int    **wght;

  void *x_thermal;  /* photon fraction; x_thermal[iv][iq1][iq2][...] */
  void *x_solar;    /* photon fraction; x_solar  [iv][iq1][iq2][...] */

  int n_comp;
} ck_struct;

/* Cross section output data STRUCTURE */

typedef struct {
  char  **filename;  /*  Filename of cross section versus wavelength, input using crs_file  */
  float ***crs;      /* 3D ary, for cross section[temperature (or layer), type, wavelength] */
  float **crs_amf;   /* 2D ary for amf calculations, needed by sdisort */
} crs_out_struct;

typedef struct {
  float** kabs; /* 2D ary, for absorption coefficient[temperature (or layer), wavelength] */
} kabs_out_struct;

typedef struct {
  double **crs;      /* cross section [layer, quadrature point] */
  double *weight;    /* weight [quadrature point]               */
  int ngauss;        /* number of quadrature points             */
  int nlev;
} ck_profile;

typedef struct {
  ck_profile **profile;      /* 2D ary, for ck_profile [type, band] */
  ck_profile ***ipa_profile; /* same as profile for different independent columns [ipa][type][band] */
} crs_ck_out_struct;


typedef struct {
  float *cf;
  float *zd;
  int nlev;
} cf_out_struct;

			    
/* INPUT STRUCTURE */

typedef struct {
  int quiet;
  int verbose;

  char         **filename;   /* array of filenames */
  
  float        wc_cloudcover;
  float        ic_cloudcover;

  int          wcipa;        /* independent pixel for water clouds */
  int          icipa;        /* independent pixel for ice clouds   */

  int          convolve;
  int          spline;
  int          header;
  int          flexstor;
  int          source;
  double       bandwidth;
  int          rayleigh;
  int          rayleigh_crs;
  float        rayleigh_depol;
  int          molabs;
  int          absorption;
  int          scattering;
  float        spline_lambda_0;
  float        spline_lambda_1;
  float        spline_lambda_step;
  int          day_of_year;
  float        latitude;     /* in seconds to avoid rounding errors*/
  float        longitude;    /* in seconds to avoid rounding errors*/
  int          ozone_spectrum;
  int          co2_spectrum;
  int          o2_spectrum;
  float        no2_column;
  int          no2_spectrum;
  int          bro_spectrum;
  int          oclo_spectrum;
  int          hcho_spectrum;
  float        precip_water;
  float        pressure;
  float        temperature;
  float        *mixing_ratio;
  int          calibration;

  int    brdf;
  float  u10;       /* wind speed for Cox and Munk parameterization */

  rte_struct     rte;
  atm_inp_struct atm;
  alt_inp_struct alt;
  alb_inp_struct alb;
  aer_inp_struct aer;

  rpv_inp_struct rpv;
  ambrals_inp_struct ambrals;
  cm_inp_struct cm;

  wl_inp_struct  wl;
  cld_inp_struct wc;
  cld_inp_struct ic;
  int            ck_scheme;
  int            ck_h2ocont;
  int            ck_o4abs;

  int            processing;
  int            output;
  int            heating;

  int    n_output_user;
  int   *output_user;     /* output_user    [column of output] = kind of output */
  int   *output_user_gas; /* output_user_gas[column of output] = MOL_GAS        */
} input_struct;


/* OUTPUT STRUCTURE */

/* elements whose memory requirements have been free'ed are listed behind */
/* each declaration. Once they are all done rm this comment.              */

typedef struct {
  float       eccentricity_factor;

  float       *column;       /* Total column of a profile, input using dens_column */
  float       *scale_factor; /* Factor to scale profile with, calculated           */
  float      **column_denstab;       /* Total column of a profile, input using dens_column [gas][sza] */
  float      **scale_factor_denstab; /* Factor to scale profile with, calculated           [gas][sza] */

  float       precip_water;
  float       water_scale_factor;

  float       *mixing_ratio;

  float       *rayleigh_depol;  /* wavelength-dependent depolarization */

  int         mc_photons;
  double      **mc_photons_t;   /* photon fraction for each wavelength and quadrature point */
  char        *mc_momcldfilename;

  float       *dtauc;           /* free'ed */
  float       *dtauc_md;        /* free'ed */
  float       *ssalb;
  float       *utau;
  float       **pmom;

  float       surface_temperature;

  /* flux, actinic flux, and intensities on the output grid */ 
  float       **rfldir;     /* direct flux                    */
  float       **rfldn;      /* diffuse downward flux          */
  float       **flup;       /* diffuse upward flux            */ 

  float       **uavgso;     /* direct actinic flux            */
  float       **uavgdn;	    /* diffuse downward actinic flux  */
  float       **uavgup;	    /* diffuse upward actinic flux    */

  float       **uavg;       /* average intensity              */
  float       ***u0u;       /* azimutally averaged intensites */
  float       ****uu;       /* intensities                    */
  float       **heat;       /* heating rates f(z,lambda)      */
  float       **w_zout;     /* vertical wind caused by radiative heating on the zout - grid */

  /* flux, actinic flux, and intensities on the internal transmittance grid */ 
  float       **rfldir_t;  /* direct flux                    */
  float       **rfldn_t;   /* diffuse downward flux          */
  float       **flup_t;	   /* diffuse upward flux            */
			                                       
  float       **uavgso_t;  /* direct actinic flux            */
  float       **uavgdn_t;  /* diffuse downward actinic flux  */
  float       **uavgup_t;  /* diffuse upward actinic flux    */
			                                       
  float       **uavg_t;	   /* average intensity              */
  float       ***u0u_t;	   /* azimutally averaged intensites */
  float       ****uu_t;	   /* intensities                    */
  float       **heat_t;    /* heating rates; depends on (z,lambda) */
  float       **w_zout_t;  /* vertical wind caused by radiative heating; depends on (z,lambda) */

  /* wavelength integrated flux, actinic flux, and intensities on the output grid */
  double      *rfldir_int;  /* direct flux                    */
  double      *rfldn_int;   /* diffuse downward flux          */
  double      *flup_int;    /* diffuse upward flux            */ 

  double      *uavgso_int;  /* direct actinic flux            */
  double      *uavgdn_int;  /* diffuse downward actinic flux  */
  double      *uavgup_int;  /* diffuse upward actinic flux    */

  double      *uavg_int;    /* average intensity              */
  double      **u0u_int;    /* azimutally averaged intensites */
  double      ***uu_int;    /* intensities                    */  

  double      *heat_int;    /* heating rates f(z) */
  double      *w_zout_int;  /* vertical wind caused by radiative heating; f(z) */

  /* integrated properties */
  double      incident;

  /* MYSTIC 3D fields on the output grid */  
  float       ****rfldir3d; 
  float       ****rfldn3d;
  float       ****flup3d;
  float       ****uavgso3d; 
  float       ****uavgdn3d; 
  float       ****uavgup3d; 
  float       ****radiance3d;
  float       ****abs3d;

  /* MYSTIC 3D fields on the internal transmittance grid */  
  float       ****rfldir3d_t;
  float       ****rfldn3d_t;
  float       ****flup3d_t;
  float       ****uavgso3d_t;
  float       ****uavgdn3d_t;
  float       ****uavgup3d_t;
  float       ****radiance3d_t;
  float       ****abs3d_t;

  /* PolRadtran flux and intensities on the output grid */ 
  float       ***down_flux;     /* free'ed */
  float       ***up_flux;       /* free'ed */
  float       *****down_rad;    /* free'ed */
  float       *****up_rad;      /* free'ed */

  /* wavelength integrated PolRadtran flux and intensities on the output grid */ 
  double      **down_flux_int;     /* free'ed */
  double      **up_flux_int;       /* free'ed */
  double      ****down_rad_int;    /* free'ed */
  double      ****up_rad_int;      /* free'ed */

  /* PolRadtran flux and intensities on the internal transmittance grid */ 
  float       ***down_flux_t;   /* free'ed */
  float       ***up_flux_t;     /* free'ed */
  float       *****down_rad_t;  /* free'ed */
  float       *****up_rad_t;    /* free'ed */


  float       *mu_values;       /* free'ed */
  float       *sza_h;           /* free'ed */

  atm_out_struct      atm;
  alt_out_struct      alt;
  alb_out_struct      alb;
  aer_out_struct      aer;
  crs_out_struct      crs;
  rpv_out_struct      rpv;
  surfaces_out_struct surf;
  kabs_out_struct     kabs;
  crs_ck_out_struct   crs_ck;

  cld_out_struct    wc;
  cld_out_struct    *wcipa;
  cld_out_struct    ic;
  cld_out_struct    *icipa;

  cf_out_struct     cf;

  mc_out_struct     mc;
  sssi_out_struct   sssi;

  float             *ipaweight;
  int               nipa;

  int               print_phi;

  wl_out_struct     wl;
  ck_struct         ck;
} output_struct;

int uvspec(input_struct input, output_struct *output);
int uvspec_check(input_struct input);


/* include all uvspec header files */
#include "albedo.h"
#include "aerosol.h"
#include "ancillary.h"
#include "atmosphere.h"
#include "ck.h"
#include "cloud.h"
#include "cloud3d.h"
#include "extraterrestrial.h"
#include "ipa.h"
#include "molecular.h"
#include "redistribute.h"
#include "solve_rte.h"
#include "sza.h"
#include "yang56.h"
#include "specrend_uvspec.h"

#if HAVE_SSS2
  #include "sss2.h"
#endif

#endif
